@@ -9,6 +9,9 @@ import warnings
 # Some sane overrides to better mix with Django
 DEBUG = getattr(settings, 'DEBUG', False) and not getattr(settings, 'SENTRY_TESTING', False)
 KEY = getattr(settings, 'SENTRY_KEY', md5_constructor(settings.SECRET_KEY).hexdigest())
+EMAIL_SUBJECT_PREFIX = getattr(settings, 'EMAIL_SUBJECT_PREFIX', EMAIL_SUBJECT_PREFIX)
+INTERNAL_IPS = getattr(settings, 'INTERNAL_IPS', INTERNAL_IPS)
+SERVER_EMAIL = getattr(settings, 'SERVER_EMAIL', SERVER_EMAIL)
 
 for k in dir(settings):
     if k.startswith('SENTRY_'):
